import Vue from "vue";
import axios from "axios";
import { Message, Loading } from "element-ui";

// token数据管理
const state = Vue.observable({
  authHttp: "",
  refreshHttp: "",
  appKey: "",
  appSecret: "",
  token: "",
  refreshToken: "",
  tokenTime: 0,
  setTime: 3000000,
  jwt: "",
  onFun: null
});

// 刷新token
export function refreshToken() {
  axios
    .get(state.refreshHttp, {
      params: {
        appKey: state.appKey,
        refreshToken: state.refreshToken
      }
    })
    .then(res => {
      if (res.status === 200 && res.data.code === 0) {
        let data = res.data.content;
        state.token = data.accessToken;
        state.refreshToken = data.refreshToken;
        state.setTime = Number(data.expire) * 1000 - 30000;
        state.tokenTime =
          new Date().getTime() + Number(data.expire) * 1000 - 30000;
        setTimeout(() => {
          refreshToken();
        }, Number(state.setTime));
      } else {
        state.setTime = 10000;
        console.log(res.data.message + ",刷新Token失败");
      }
    })
    .catch(error => {
      state.setTime = 10000;
      console.log(error + ",刷新Token失败");
    });
}

// http加密封装
export function httpEncrypt(data) {
  let timeStamp = new Date().getTime();
  const newjson = JSON.parse(
    JSON.stringify(
      Object.assign(data, {
        appKey: state.appKey,
        accessToken: state.token,
        time: timeStamp.toString()
      })
    )
  );
  return {
    appKey: state.appKey,
    accessToken: state.token,
    time: timeStamp.toString(),
    sign: onSign(newjson, state.appSecret)
  };
}

// 获取token
async function initToken(data) {
  return new Promise((resolve, reject) => {
    let nowTime = new Date().getTime();
    if (nowTime > state.tokenTime) {
      let httpUrl = state.refreshHttp;
      let httpData = {
        appKey: state.appKey
      };
      if (state.tokenTime == 0) {
        httpUrl = state.authHttp;
        httpData.appSecret = state.appSecret;
      } else {
        httpData.refreshToken = state.refreshToken;
      }
      axios
        .get(httpUrl, {
          params: httpData
        })
        .then(res => {
          if (res.status === 200) {
            state.token = res.data.content.accessToken;
            state.refreshToken = res.data.content.refreshToken;
            state.setTime = Number(res.data.content.expire) * 1000 - 30000;
            state.tokenTime =
              new Date().getTime() + Number(data.expire) * 1000 - 30000;
            if (state.onFun != null) {
              clearTimeout(state.onFun);
            }
            state.onFun = window.setTimeout(() => {
              refreshToken();
            }, Number(state.setTime));
            resolve(httpEncrypt(data));
          } else {
            Message.error("获取token失败");
            reject(res.data.message);
          }
        })
        .catch(error => {
          Message.error("服务器请求失败");
          reject(error);
        });
    } else {
      resolve(httpEncrypt(data));
    }
  });
}

// 加密
function onSign(obj, appSecret) {
  delete obj.requestId;
  delete obj.sign;
  const keys = Object.keys(obj).sort();
  let pinStr = "";
  for (const key of keys) {
    pinStr +=
      typeof obj[key] === "object" ? JSON.stringify(obj[key]) : obj[key];
  }
  pinStr += appSecret;
  const signStr = require("crypto")
    .createHash("SHA256")
    .update(pinStr)
    .digest("HEX")
    .toUpperCase();
  return signStr;
}

const api = {
  // 初始化
  create(obj) {
    //设置请求基地址
    axios.defaults.baseURL = obj.baseURL ? obj.baseURL : "";
    //设置超时
    axios.defaults.timeout = obj.timeout ? obj.timeout : 10000;
    // token相关
    state.authHttp = obj.authURL ? obj.authURL : 'http://www.dalitek.tech:12000/main/v1.0.0/auth';
    state.refreshHttp = obj.refreshURL ? obj.refreshURL : 'http://www.dalitek.tech:12000/main/v1.0.0/auth/refresh';
    state.appKey = obj.appKey;
    state.appSecret = obj.appSecret;
  },
  // jwt
  jwt(jwt) {
    // 用户登录jwt
    state.jwt = jwt;
  },
  // get
  async get(url, getData = {}) {
    let params = JSON.parse(JSON.stringify(getData));
    let loadingInstance = null;
    let timeoutLoading = window.setTimeout(() => {
      loadingInstance = Loading.service({ fullscreen: true });
    }, 300);
    let newData = JSON.parse(JSON.stringify(params));
    let resData = await initToken(params);
    return new Promise((resolve, reject) => {
      axios
        .get(url, {
          headers: {
            Authorization: "Bearer " + state.jwt
          },
          params: Object.assign(newData, resData)
        })
        .then(
          response => {
            clearTimeout(timeoutLoading);
            if (loadingInstance != null) {
              loadingInstance.close();
            }
            if (response.status === 200 && response.data.code === 0) {
              resolve(response.data.content);
            } else {
              if (response.data) {
                Message.error(response.data.message);
              } else {
                Message.error("服务器请求失败");
              }
              reject(response.data.message);
            }
          },
          error => {
            clearTimeout(timeoutLoading);
            if (loadingInstance != null) {
              loadingInstance.close();
            }
            Message.error("服务器请求失败");
            reject(error);
          }
        )
        .catch(error => {
          console.log(error);
        });
    });
  },
  // delete
  async delete(url, deleteData = {}) {
    let params = JSON.parse(JSON.stringify(deleteData));
    let loadingInstance = null;
    let timeoutLoading = window.setTimeout(() => {
      loadingInstance = Loading.service({ fullscreen: true });
    }, 300);
    let newData = JSON.parse(JSON.stringify(params));
    let resData = await initToken(params);
    return new Promise((resolve, reject) => {
      axios
        .delete(url, {
          headers: {
            Authorization: "Bearer " + state.jwt
          },
          params: Object.assign(newData, resData)
        })
        .then(
          response => {
            clearTimeout(timeoutLoading);
            if (loadingInstance != null) {
              loadingInstance.close();
            }
            if (response.status === 200 && response.data.code === 0) {
              resolve(response.data.content);
            } else {
              if (response.data) {
                Message.error(response.data.message);
              } else {
                Message.error("服务器请求失败");
              }
              reject(response.data.message);
            }
          },
          error => {
            clearTimeout(timeoutLoading);
            if (loadingInstance != null) {
              loadingInstance.close();
            }
            Message.error("服务器请求失败");
            reject(error);
          }
        )
        .catch(error => {
          console.log(error);
        });
    });
  },
  // post
  async post(url, postData = {}) {
    let data = JSON.parse(JSON.stringify(postData));
    let loadingInstance = null;
    let timeoutLoading = window.setTimeout(() => {
      loadingInstance = Loading.service({ fullscreen: true });
    }, 300);
    let newData = JSON.parse(JSON.stringify(data));
    let resData = await initToken(data);
    return new Promise((resolve, reject) => {
      axios
        .post(
          url,
          {
            header: resData,
            payload: newData
          },
          {
            headers: {
              Authorization: "Bearer " + state.jwt
            }
          }
        )
        .then(
          response => {
            clearTimeout(timeoutLoading);
            if (loadingInstance != null) {
              loadingInstance.close();
            }
            if (response.status === 200 && response.data.code === 0) {
              resolve(response.data.content);
            } else {
              if (response.data) {
                Message.error(response.data.message);
              } else {
                Message.error("服务器请求失败");
              }
              reject(response.data.message);
            }
          },
          error => {
            clearTimeout(timeoutLoading);
            if (loadingInstance != null) {
              loadingInstance.close();
            }
            Message.error("服务器请求失败");
            reject(error);
          }
        )
        .catch(error => {
          console.log(error);
        });
    });
  },
  // put
  async put(url, putData = {}) {
    let data = JSON.parse(JSON.stringify(putData));
    let loadingInstance = null;
    let timeoutLoading = window.setTimeout(() => {
      loadingInstance = Loading.service({ fullscreen: true });
    }, 300);
    let newData = JSON.parse(JSON.stringify(data));
    let resData = await initToken(data);
    return new Promise((resolve, reject) => {
      axios
        .put(
          url,
          {
            header: resData,
            payload: newData
          },
          {
            headers: {
              Authorization: "Bearer " + state.jwt
            }
          }
        )
        .then(
          response => {
            clearTimeout(timeoutLoading);
            if (loadingInstance != null) {
              loadingInstance.close();
            }
            if (response.status === 200 && response.data.code === 0) {
              resolve(response.data.content);
            } else {
              if (response.data) {
                Message.error(response.data.message);
              } else {
                Message.error("服务器请求失败");
              }
              reject(response.data.message);
            }
          },
          error => {
            clearTimeout(timeoutLoading);
            if (loadingInstance != null) {
              loadingInstance.close();
            }
            Message.error("服务器请求失败");
            reject(error);
          }
        )
        .catch(error => {
          console.log(error);
        });
    });
  }
};

export default api;
